#version 300 es

precision highp float;

layout(location = 0) out vec4 F;
in vec2 v_tex_coord;

uniform sampler2D texture_tfm;
uniform sampler2D texture_fdog_h;

uniform float phi;

struct lic_t 
{ 
    vec2 p; 
    vec2 t;
    float w;
    float dw;
};

void step1(inout lic_t s) 
{
    vec2 t = texture(texture_tfm, s.p).xy;
    t = vec2(t.y, -t.x);

    t *= sign(dot(t, s.t));
    
    s.t = t;

    s.dw = (abs(t.x) > abs(t.y)) 
            ? abs((fract(s.p.x) - 0.5 - sign(t.x)) / t.x) 
            : abs((fract(s.p.y) - 0.5 - sign(t.y)) / t.y);

    vec2 textire_size_inv = 1.0 / vec2(textureSize(texture_fdog_h, 0));
    s.p += t * s.dw * textire_size_inv;
    s.w += s.dw;
}

void main() 
{
    float sigma_m = 2.0;
    float phi = 3.5;
    float two_sigma_m_squared = 2.0 * sigma_m * sigma_m;
    float half_width = 5.0 * sigma_m;

    vec2 uv = v_tex_coord;
    vec2 textire_size = vec2(textureSize(texture_fdog_h, 0));

    float H = texture(texture_fdog_h, uv).x;
    float w = 1.0;

    lic_t a, b;
    a.p = b.p = uv;
    vec2 t = texture(texture_tfm, uv).xy;
    t = vec2(t.y, -t.x);
    a.t =  t / textire_size;
    b.t = -a.t;
    a.w = b.w = 0.0;

    while (a.w < half_width) {
        step1(a);
        float k = a.dw * exp(-a.w * a.w / two_sigma_m_squared);
        H += k * texture(texture_fdog_h, a.p).x;
        w += k;
    }

    while (b.w < half_width) {
        step1(b);
        float k = b.dw * exp(-b.w * b.w / two_sigma_m_squared);
        H += k * texture(texture_fdog_h, b.p).x;
        w += k;
    }

    H /= w;

    float edge = (H > 0.0) ? 0.0 : 1.0;
    F = vec4(edge, 1.0, 1.0, 1.0);
}
